@@ -4,8 +4,7 @@
 #
 # DeVeDe-NG is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
-# (at your option) any later version.
+# the Free Software Foundation; either version 3 of the License.
 #
 # DeVeDe-NG is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,6 +33,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
 
     def __init__(self, file_name, list_files=None):
 
+        self.element_type = "file_movie"
         self.list_files = list_files
 
         devedeng.interface_manager.interface_manager.__init__(self)
@@ -73,7 +73,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
         self.add_toggle("copy_sound", False)
         self.add_toggle("is_mpeg_ps", False)
         self.add_toggle("no_reencode_audio_video", False)
-        if (self.disc_type == "divx") or (self.disc_type == "mkv"):
+        if (self.disc_type == "divx") or (self.disc_type == "mkv") or (self.config.disc_type == "hevc"):
             self.add_toggle("gop12", False)
         else:
             self.add_toggle("gop12", True)
@@ -93,6 +93,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
         self.add_group("actions", ["action_stop", "action_play_first", "action_play_previous",
                                    "action_play_again", "action_play_next", "action_play_last"], "action_stop")
 
+        self.prerotation = 0
         self.add_integer_adjustment("volume", 100)
         if (self.disc_type == "dvd"):
             self.add_integer_adjustment("video_rate", 5000)
@@ -174,6 +175,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
                 self.original_aspect_ratio = film_analizer.original_aspect_ratio
                 self.original_videorate = film_analizer.original_videorate
                 self.original_audiorate = film_analizer.original_audiorate
+                self.prerotation = film_analizer.prerotation
 
                 self.original_audiorate_uncompressed = film_analizer.original_audiorate_uncompressed
                 self.original_fps = film_analizer.original_fps
@@ -201,7 +203,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
             self.aspect_ratio_final = None
             self.converted_filename = None
 
-    def set_title(self, new_title):
+    def set_name(self, new_title):
         self.title_name = new_title
         self.emit('title_changed', self.title_name)
 
@@ -248,6 +250,8 @@ class file_movie(devedeng.interface_manager.interface_manager):
 
     def get_max_resolution(self, rx, ry, aspect):
 
+        if aspect == 0:
+            aspect = 1
         tmpx = ry * aspect
         tmpy = rx / aspect
         if (tmpx > rx):
@@ -257,7 +261,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
 
     def set_final_rates(self):
 
-        if (self.disc_type == "divx") or (self.disc_type == "mkv"):
+        if (self.disc_type == "divx") or (self.disc_type == "mkv") or (self.config.disc_type == "hevc"):
             self.audio_rate_auto = 192
         elif (self.disc_type == "vcd") or (self.disc_type == "svcd") or (self.disc_type == "cvd"):
             self.audio_rate_auto = 224
@@ -302,7 +306,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
 
         # for divx or matroska, if the size and the aspect ratio is automatic,
         # just don't change them
-        if ((self.disc_type == "divx") or (self.disc_type == "mkv")) and (final_size == "size_auto") and (self.aspect_ratio == "aspect_auto"):
+        if ((self.disc_type == "divx") or (self.disc_type == "mkv") or (self.config.disc_type == "hevc")) and (final_size == "size_auto") and (self.aspect_ratio == "aspect_auto"):
             self.width_midle = self.original_width
             self.width_final = self.original_width
             self.height_midle = self.original_height
@@ -364,8 +368,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
                         else:
                             self.height_final = 480
                     else:
-                        tx, ty = self.get_max_resolution(
-                            self.original_width, self.original_height, self.original_aspect_ratio)
+                        tx, ty = self.get_max_resolution(self.original_width, self.original_height, self.original_aspect_ratio)
                         if (self.format_pal):
                             th = 576
                             th2 = 288
@@ -385,8 +388,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
                             self.width_final = 720
                             self.height_final = th
                 else:
-                    self.width_final, self.height_final = self.get_max_resolution(
-                        self.original_width, self.original_height, self.original_aspect_ratio)
+                    self.width_final, self.height_final = self.get_max_resolution(self.original_width, self.original_height, self.original_aspect_ratio)
             else:
                 values = final_size[5:].split("x")
                 self.width_final = int(values[0])
@@ -399,46 +401,46 @@ class file_movie(devedeng.interface_manager.interface_manager):
 
         if (self.rotation == "rotation_90") or (self.rotation == "rotation_270"):
             midle_aspect_ratio = 1.0 / self.original_aspect_ratio
+            original_width = self.original_height
+            original_height = self.original_width
         else:
+            original_width = self.original_width
+            original_height = self.original_height
             midle_aspect_ratio = self.original_aspect_ratio
 
+        if midle_aspect_ratio == 0:
+            midle_aspect_ratio = 1
+
         if self.scaling == "scale_picture":
-            self.width_midle = int(self.original_width)
-            self.height_midle = int(self.original_height)
+            self.width_midle = int(original_width)
+            self.height_midle = int(original_height)
         elif self.scaling == "add_black_bars":
+            print(f"Original width {original_width}x{original_height}; ratio original: {midle_aspect_ratio}; ratio nuevo: {self.aspect_ratio_final}")
             if midle_aspect_ratio > self.aspect_ratio_final:  # add horizontal black bars, at top and bottom
-                self.width_midle = int(self.original_width)
-                self.height_midle = int(
-                    self.original_height * midle_aspect_ratio / self.aspect_ratio_final)
+                self.width_midle = int(original_width)
+                self.height_midle = int(original_height * midle_aspect_ratio / self.aspect_ratio_final)
             else:  # add vertical black bars, at left and right
-                self.width_midle = int(
-                    self.original_width * self.aspect_ratio_final / midle_aspect_ratio)
-                self.height_midle = int(self.original_height)
+                self.width_midle = int(original_width * self.aspect_ratio_final / midle_aspect_ratio)
+                self.height_midle = int(original_height)
         else:  # cut picture
             if midle_aspect_ratio > self.aspect_ratio_final:
-                self.width_midle = int(
-                    self.original_width * self.aspect_ratio_final / midle_aspect_ratio)
-                self.height_midle = int(self.original_height)
+                self.width_midle = int(original_width * self.aspect_ratio_final / midle_aspect_ratio)
+                self.height_midle = int(original_height)
             else:
-                self.width_midle = int(self.original_width)
-                self.height_midle = int(
-                    self.original_height * midle_aspect_ratio / self.aspect_ratio_final)
+                self.width_midle = int(original_width)
+                self.height_midle = int(original_height * midle_aspect_ratio / self.aspect_ratio_final)
 
     def set_type(self, obj=None, disc_type=None):
-
         if (disc_type is not None):
             self.disc_type = disc_type
 
-    def delete_file(self):
-
+    def delete_element(self):
         return
 
     def on_help_clicked(self, b):
-
         help_file = devedeng.help.help("file.html")
 
     def properties(self):
-
         if (self.wfile_properties is not None):
             self.wfile_properties.present()
             return
@@ -473,7 +475,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
         self.wframe_division_chapters = self.builder.get_object(
             "frame_division_chapters")
 
-        if (self.disc_type == "dvd") or (self.disc_type == "divx") or (self.disc_type == "mkv"):
+        if (self.disc_type == "dvd") or (self.disc_type == "divx") or (self.disc_type == "mkv") or (self.config.disc_type == "hevc"):
             self.waudio_rate.set_upper(448.0)
         else:
             self.waudio_rate.set_upper(384.0)
@@ -556,7 +558,7 @@ class file_movie(devedeng.interface_manager.interface_manager):
             self.wcopy_sound.hide()
             self.wnotebook.remove_page(5)
             self.wnotebook.remove_page(1)
-        elif (self.disc_type == 'mkv'):
+        elif (self.disc_type == 'mkv') or (self.config.disc_type == "hevc"):
             self.wshow_in_menu.hide()
             self.wframe_division_chapters.hide()
             self.wnotebook.remove_page(5)
@@ -624,12 +626,12 @@ class file_movie(devedeng.interface_manager.interface_manager):
             self.emit('in_menu_changed', self.show_in_menu)
         else:
             # editing properties for a group of files
-            data = self.store_file()
+            data = self.store_element()
             sel = self.wtreeview_multiproperties.get_selection()
             model, pathlist = sel.get_selected_rows()
             for file_path in pathlist:
                 obj = model[file_path][1]
-                obj.restore_file(data)
+                obj.restore_element(data)
 
         self.wfile_properties.destroy()
         self.wfile_properties = None
@@ -744,24 +746,20 @@ class file_movie(devedeng.interface_manager.interface_manager):
             disc_player.play_film(os.path.join(
                 self.config.tmp_folder, "movie_preview.mpg"))
 
-    def store_file(self):
-
+    def store_element(self):
         data = self.serialize()
         if "files_to_set" in data:
             del data["files_to_set"]
-
+        data["element_type"] = "file_movie"
         return data
 
-    def restore_file(self, data):
-
+    def restore_element(self, data):
         self.unserialize(data)
 
     def on_select_all_clicked(self, b):
-
         sel = self.wtreeview_multiproperties.get_selection()
         sel.select_all()
 
     def on_unselect_all_clicked(self, b):
-
         sel = self.wtreeview_multiproperties.get_selection()
         sel.unselect_all()
