@@ -6,8 +6,7 @@
 #
 # DeVeDe-NG is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 3 of the License, or
-# (at your option) any later version.
+# the Free Software Foundation; either version 3 of the License.
 #
 # DeVeDe-NG is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,6 +19,7 @@
 from gi.repository import GObject
 import os
 import pkg_resources
+import platform
 
 
 class configuration(GObject.GObject):
@@ -31,7 +31,7 @@ class configuration(GObject.GObject):
     def get_config():
         if configuration.current_configuration is None:
             configuration.current_configuration = configuration()
-            if (configuration.current_configuration.fill_config()):
+            if (configuration.current_configuration._fill_config()):
                 configuration.current_configuration = None
         return configuration.current_configuration
 
@@ -40,11 +40,21 @@ class configuration(GObject.GObject):
         self.version = str(pkg_resources.require("devedeng")[0].version)
         print("Version: " + self.version)
 
-    def fill_config(self):
+    def _get_config_folder(self):
+        config_folder = os.environ.get("$XDG_CONFIG_HOME")
+        if config_folder is None:
+            config_folder = os.path.join(os.environ.get("HOME"), ".config")
+        config_folder = os.path.join(config_folder, "devedeng")
+        if not os.path.exists(config_folder):
+            os.makedirs(config_folder)
+        return config_folder
 
-        self.cores = 0
-        if (os.name == "posix"):
-            self.cores = os.cpu_count()
+    def _fill_config(self):
+        self.cores = os.cpu_count()
+        if 'sched_getaffinity' in dir(os):
+            self.cores = len(os.sched_getaffinity(0))
+        if platform.system() == 'FreeBSD':
+            self.cores = int(os.popen("sysctl -n kern.smp.cores").read().split('\n')[0])
 
         is_local = None
         self.log = ""
@@ -105,7 +115,12 @@ class configuration(GObject.GObject):
         self.subt_outline_color = (0, 0, 0, 1)
         self.subt_outline_thickness = 0.0
 
-        config_path = os.path.join(os.environ.get("HOME"), ".devedeng")
+        config_folder = self._get_config_folder()
+        config_path = os.path.join(config_folder, "devedeng.cfg")
+        if not os.path.exists(config_path):
+            # old config file
+            config_path = os.path.join(os.environ.get("HOME"), ".devedeng")
+
         try:
             config_data = open(config_path, "r")
             for linea in config_data:
@@ -181,7 +196,7 @@ class configuration(GObject.GObject):
 
     def save_config(self):
 
-        config_path = os.path.join(os.environ.get("HOME"), ".devedeng")
+        config_path = os.path.join(self._get_config_folder(), "devedeng.cfg")
         try:
             config_data = open(config_path, "w")
             config_data.write("video_format:")
@@ -233,6 +248,10 @@ class configuration(GObject.GObject):
                               str(self.subt_outline_thickness))
 
             config_data.close()
+            # remove old configuration files
+            config_path = os.path.join(os.environ.get("HOME"), ".devedeng")
+            if os.path.exists(config_path):
+                os.remove(config_path)
         except:
             pass
 
